''' Here we will use the simplified version of 'rich comparison.' We only need
to define __eq__ and __ne__ followed by any one of the remaining 4 (lt, le, gt, ge)
and then the command @total_ordering decorator will automatically define the rest
'''

from functools import total_ordering

@total_ordering

class Card1():
    """Represents a standard playing card.
    
    Attributes:
      suit: integer 0-3
      rank: integer 1-13
    """

    # These are 'global attributes': Constant values that will remain the same
    # for all Card1 objects... We can refer to these lists as Card1.suit_names, etc.
    suit_names = ["Clubs", "Diamonds", "Hearts", "Spades"]
    rank_names = [None, "Ace", "2", "3", "4", "5", "6", "7", 
              "8", "9", "10", "Jack", "Queen", "King"]

    def __init__(self, suit=0, rank=2):
        self.suit = suit
        self.rank = rank

    def __str__(self):
        """Returns a human-readable string representation."""
        return '{} of {}'.format(Card1.rank_names[self.rank],
                             Card1.suit_names[self.suit])



    """__cmp__ of Python 2 was replaced by 'rich comparison' methods to increase
    robustness of comparing two objects. This now requires us to define all 6 of 
    the following functions
    ==	__eq__
    !=	__ne__
    <	__lt__
    <=	__le__
    >	__gt__
    >=	__ge__
    """
    
    def __eq__(self, other):
        ''' 2 cards are only equal when both suit and rank are the same'''
        t1 = (self.suit, self.rank)
        t2 = (other.suit, other.rank)
        return (t1==t2)

    def __ne__(self, other):
        ''' return 'not equal' if either suit or rank (or both) is different'''
        t1 = (self.suit, self.rank)
        t2 = (other.suit, other.rank)
        return (t1 != t2)

    def __lt__(self, other):
        ''' return that self < other is True if self.suit < other.suit or if 
        the suits are equal, then self.rank < other.rank'''
        t1 = (self.suit, self.rank)
        t2 = (other.suit, other.rank)
        return (t1 < t2)

    # we can stop here: no need to define the other 3 based on how we've defined lt
